//
//  CalculatorController.swift
//  calculator
//
//  Created by Илья Лошкарёв on 18.02.17.
//  Copyright © 2017 Илья Лошкарёв. All rights reserved.
//

import UIKit

class CalculatorController: UIViewController {
    
    let buttonText = [["C", "±", "%", "÷"],
                      ["7", "8", "9", "×"],
                      ["4", "5", "6", "-"],
                      ["1", "2", "3", "+"],
                      ["0", "",  ".", "="]]
    
    var outputLabel: UILabel!
    
    weak var calculator: Calculator?
    
    let formatter = NumberFormatter()
    let maximumFractionDigits = 5
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        initUI()
        
        // Обращаемся к созданному калькулятору
        calculator = (UIApplication.shared.delegate as! AppDelegate).calculator
        // Становимся представителем
        calculator?.delegate = self
        // Очищаем состояние калькулятора
        calculator?.reset();
        
        // Число с одной цифрой
        formatter.minimumIntegerDigits = 1
    }
    
    func initUI() {
        let mainStack = UIStackView()
        mainStack.axis = .vertical
        mainStack.alignment = .fill
        mainStack.distribution = .fillEqually
        
        // AUTO LAYOUT
        view.addSubview(mainStack)
        mainStack.translatesAutoresizingMaskIntoConstraints = false
        let constraints = [
            NSLayoutConstraint (item: mainStack, attribute: .leading,
                                relatedBy: .equal,
                                toItem: view.safeAreaLayoutGuide, attribute: .leading,
                                multiplier: 1, constant: 0),
            NSLayoutConstraint (item: mainStack, attribute: .trailing,
                                relatedBy: .equal,
                                toItem: view.safeAreaLayoutGuide, attribute: .trailing,
                                multiplier: 1, constant: 0),
            NSLayoutConstraint (item: mainStack, attribute: .top,
                                relatedBy: .equal,
                                toItem: view.safeAreaLayoutGuide, attribute: .top,
                                multiplier: 1, constant: 0),
            NSLayoutConstraint (item: mainStack, attribute: .bottom,
                                relatedBy: .equal,
                                toItem: view.safeAreaLayoutGuide, attribute: .bottom,
                                multiplier: 1, constant: 0)
        ]
        
        NSLayoutConstraint.activate(constraints)
        
        // LABEL
        outputLabel = UILabel()
        outputLabel.text = String()
        outputLabel.textAlignment = .right
        outputLabel.font = UIFont(name: "Menlo-Regular", size: 28)
        
        mainStack.addArrangedSubview(outputLabel)

        // BUTTONS
        for i in 0..<buttonText.count {
            let buttonStack = UIStackView()
            buttonStack.axis = .horizontal
            buttonStack.alignment = .fill
            buttonStack.distribution = .fillEqually
            
            for j in 0..<buttonText[i].count {
                let button = createGridButton(row: i, col: j)
                button.addTarget(self, action: #selector(buttonTouched(sender:)), for: .touchUpInside)
                buttonStack.addArrangedSubview(button)
            }
            mainStack.addArrangedSubview(buttonStack)
        }
        

    }
    
    func createGridButton(row: Int, col: Int) -> UIButton {
        
        let button = UIButton(type: .system)

        if buttonText[row][col] == "." {
            // Localized separator
            button.setTitle(formatter.decimalSeparator, for: .normal)
        } else {
            button.setTitle(buttonText[row][col], for: .normal)
        }
        
        button.titleLabel?.font = outputLabel.font.withSize(28)
        return button
    }
    
    @objc func buttonTouched(sender: UIButton)  {
        
        guard let content = sender.titleLabel?.text else {
            print("No text for touched button")
            return
        }
        
        switch content {
            
        case "0"..."9":
            calculator?.addDigit(Int(content)!)
            
        case formatter.decimalSeparator:
            calculator?.addPoint()
            
        case "C":
            if let _ = calculator?.input {
                calculator?.clear();
            } else {
                calculator?.reset();
            }
            
        case "%", "±", "+", "-", "×", "÷":
            calculator?.addOperation(Operation(rawValue:content)!)
            
        case "=":
            calculator?.compute()
            
        default:
            print("Unknown button")
        }
    }
}

