//
//  Calculator.swift
//  calculator
//
//  Created by Илья Лошкарёв on 21.09.17.
//  Copyright © 2017 Илья Лошкарёв. All rights reserved.
//

import Foundation


/// Доступные операции
enum Operation: String {
    case add = "+",
    sub = "-",
    mul = "×",
    div = "÷",
    // унарные операции
    sign = "±", // (input * -1)
    perc = "%"  // (input / 100)
}


/// Протокол калькулятора
protocol Calculator: class { // можно реализовывать только в ссылочном типе
    
    /// Представитель – объект реагирующий на изменение внутреннего состояния калькулятора
    var delegate: CalculatorDelegate? { get set }
    
    /// Инициализатор
    /// `len` – максимальное количество символов в поле ввода
    /// `frac` – максимальное количество знаков после запятой
    init(inputLength len: UInt, fractionLength frac: UInt)
    
    // Хранимое выражение: <левое значение> <операция> <правое значение>
    
    /// Левое значение
    var result: Double? { get }
    
    /// Текущая операция
    var operation: Operation? { get }
    
    /// Правое значение
    var input: Double? { get }
    
    /// Добавить цифру к правому значению
    func addDigit(_ d: Int)
    
    /// Добавить точку к правому значению
    func addPoint()
    
    /// Правое значение содержит точку
    var hasPoint: Bool { get }
    
    /// Количество текущих знаков после запятой в правом значении
    var fractionDigits: UInt { get }
    
    /// Добавить операцию
    /// Если операция унарная, применить её к правому значению.
    /// Если левое значение не заданно, перенести правое значение влево.
    /// Если текущая операция уже задана, вычислить значение выражения и
    /// добавить операцию к результату.
    func addOperation(_ op: Operation)
    
    /// Вычислить значение выражения
    func compute()
    
    /// Очистить правое значение
    func clear()
    
    /// Очистить выражение
    func reset()
}

