//
//  Calculator.swift
//  calculator
//
//  Created by Илья Лошкарёв on 20.09.17.
//  Copyright © 2017 Илья Лошкарёв. All rights reserved.
//

import Foundation

/// Протокол представителя для калькулятора
protocol CalculatorDelegate: class {
    
    /// Калькулятор обновил поле ввода
    /// `calculator` - ссылка на объект
    /// `value` - новое значение
    /// `fractionDigits` - количество значащих цифр в значении
    func calculatorDidUpdateValue(_ calculator: Calculator, with value: Double, valuePrecision fractionDigits: UInt)
    
    /// Калькулятор очистил хранимые значения, установив значения по умолчанию
    /// `calculator` - ссылка на объект
    /// `value` - значение по умолчанию
    /// `fractionDigits` - количество значащих цифр в значении
    func calculatorDidClear(_ calculator: Calculator, withDefaultValue value: Double?, defaultPrecision fractionDigits: UInt?)
    
    // MARK: Ошибки
    
    /// Переполнение поля ввода
    /// Количество значащих цифр превышает допустимый предел
    func calculatorDidInputOverflow(_ calculator: Calculator)
    
    /// Ошибка вычислений
    /// Результат операции не представим вещественным числом
    func calculatorDidNotCompute(_ calculator: Calculator, withError message: String)
}
